{ Copyright (C) 2001 suxxx@users.sourceforge.net
This is free software distributed under the terms of the
GNU Public License.  See the file COPYING for details.
$Id: dc_client_server.pas,v 0.1 2002/01/03 16:30 suxxx $
}

{ here all client to server interactions stored }

{ to use debug messages }
//{$DEFINE DEBUG}
//{$DEFINE USEDCWELCOME}
unit dc_client_server;

interface
uses
     { kylix / delphi units }
     SysUtils,
     classes,
     { indy units }
     IdIPWatch,
     IdBaseComponent,
     IdComponent,
     IdSocketHandle,
     IdTCPServer,
     IdGlobal,
     IdIOHandlerSocket,
     { opendcd units }
     declare_types;
     { more units will be used after the implementation in custom classes } 
     
type
  TDCHub = class(TComponent)
  protected
   {hub settings}
    _HubName:smallstring;
    _HubMaxUsers:integer;
    _HubMinFileShare:int64;
    _HubRedirectAddress:middlestring;
    _HubRedirectAllUsers:boolean;
    _HubRedirectIfHubIsFull:boolean;
    _HubIsRegisteredUsersOnly:boolean;
    _HubDisablePassiveSearch:boolean;
    _HubUsers:TConnectedUserList;
    _HubTCPListener:TIdTCPServer;
    procedure _HubTCPListenerSetActive(Value: boolean); virtual;
    function _HubTCPListenerGetActive():boolean; virtual;
  private
    { _HubTCPListener events }
    procedure TCPConnectionClose(AThread: TIdPeerThread);
    procedure TCPConnectionOpen(AThread: TIdPeerThread);
    procedure TCPExecute(AThread: TIdPeerThread);
    { DCLinkedHub events }
    procedure DCLNK_Search(SearchRequest:string);
    procedure DCLNK_ConnectToMe(ConnectToMeString:string);
    { Increase / Decrease Function for TConnectedUserArray }
    function AddConnectedUser(socket:TIdPeerThread):boolean;
    function DelConnectedUser(socket:TIdPeerThread):boolean;
    { Main Message Parse for the User }
    procedure ODC_UserMessageParse(socket:TIdPeerThread;ReceivedData:string);
    procedure ODC_UserConnected(socket:TIdPeerThread);
    procedure ODC_UserDisconnected(socket:TIdPeerThread);
    { DCMSG Handling}
    procedure DCMSG_ValidateNick(socket:TIdPeerThread;msg:string);
    procedure DCMSG_MyPass(socket:TIdPeerThread;msg:string);
    procedure DCMSG_MyINFO(socket:TIdPeerThread;msg:string);
    procedure DCMSG_GetNickList(socket:TIdPeerThread;msg:string);
    procedure DCMSG_GetINFO(socket:TIdPeerThread;msg:string);
    procedure DCMSG_To(socket:TIdPeerThread;msg:string);
    procedure DCMSG_Key(socket:TIdPeerThread;msg:string);
    procedure DCMSG_Version(socket:TIdPeerThread;msg:string);
    procedure DCMSG_Quit(socket:TIdPeerThread;msg:string);
    procedure DCMSG_ConnectToMe(socket:TIdPeerThread;msg:string);
    procedure DCMSG_MultiConnectToMe(socket:TIdPeerThread;msg:string);
    procedure DCMSG_RevConnectToMe(socket:TIdPeerThread;msg:string);
    procedure DCMSG_SR(socket:TIdPeerThread;msg:string);
    procedure DCMSG_Search(socket:TIdPeerThread;msg:string);
    procedure DCMSG_MultiSearch(socket:TIdPeerThread;msg:string);
    procedure DCMSG_Kick(socket:TIdPeerThread;msg:string);
    procedure DCMSG_OpForceMove(socket:TIdPeerThread;msg:string);
  public
   { functions / procedure declarations }
    constructor Create(ipaddress:string;port:integer);
    destructor Destroy();
    { DCFunctions }
    function DCCMD_SendBroadCastMsg(msg:string):boolean;
    function DCCMD_RedirectAllUsers():boolean;
    function DCCMD_KickAllUsers():boolean;
    function DCCMD_SendToUserMsg(socket:TIdPeerThread;msg:string):boolean;
    function DCCMD_GetSocketbyUsername(username:smallstring): TIdPeerThread;
    function DCCMD_GetUsernamebySocket(socket:TIdPeerThread):smallstring;
    function DCCMD_GetArrayOffsetbySocket(socket:TIdPeerThread): integer;
    function DCCMD_IsOnline(username:string):boolean;
    procedure DCCMD_SendNicklist(socket:TIdPeerThread = nil);
    procedure DCCMD_SendOperatorlist(socket:TIdPeerThread = nil);
    function DCCMD_KickUser(socket:TIdPeerThread;msg:string = ''):boolean;
    function DCCMD_RedirectUser(socket:TIdPeerThread):boolean;
    procedure DCCMD_SendMOTD(socket:TIdPeerThread);
   { properties }
    property HubName:smallstring read _HubName write _HubName;
    property HubMaxUsers:integer read _HubMaxUsers write _HubMaxUsers;
    property HubMinFileShare:int64 read _HubMinFileShare write _HubMinFileShare;
    property HubRedirectAddress:middlestring read _HubRedirectAddress write _HubRedirectAddress;
    property HubRedirectAllUsers:boolean read _HubRedirectAllUsers write _HubRedirectAllUsers;
    property HubRedirectIfHubIsFull:boolean read _HubRedirectIfHubIsFull write _HubRedirectIfHubIsFull;
    property HubIsRegisteredUsersOnly:boolean read _HubIsRegisteredUsersOnly write _HubIsRegisteredUsersOnly;
    property HubDisablePassiveSearch:boolean read _HubDisablePassiveSearch write _HubDisablePassiveSearch;
    property HubUsers:TConnectedUserList read _HubUsers;

    property HubTCPListenerActive:boolean read _HubTCPListenerGetActive write _HubTCPListenerSetActive;


   end;

implementation

{ all opendcd units will be used after the implementation }
{ only in this class because i need a global DCHub Variable }
uses declare_vars,
     dc_server_server,
     global_functions,
     global_reguser_functions,
     global_services,
     global_ban_functions,
     global_memo_functions,
     global_multiconsol_functions;

var  DCLinkedHub:TDCLinkedHub;

{ *** TComponent Functions / Procedures ***}

{ TDCHub creation procedure }
constructor TDCHub.Create(ipaddress:string;port:integer);
var socket : TIdSocketHandle;
begin
  inherited Create(nil);

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Create] Create executed');
  {$ENDIF}



  {$IFDEF DEBUG}
   writeln ('[TDCHub.Create] createing Server socket');
  {$ENDIF}
  { creating the tcp server socket }
  _HubTCPListener := TIdTCPServer.Create(nil);
  socket := _HubTCPListener.Bindings.Add;
  socket.IP := ipaddress;
  socket.Port := port;

  { linking the events of _HubTCPListener with the procedures }
  _HubTCPListener.OnDisconnect := TCPConnectionClose;
  _HubTCPListener.OnConnect := TCPConnectionOpen;
  _HubTCPListener.OnExecute := TCPExecute;


  {$IFDEF DEBUG}
   writeln ('[TDCHub.Create] createing LinkingHub');
  {$ENDIF}
  try
   DCLinkedHub := TDCLinkedHub.Create(ipaddress,port);
  except
   //raise;
  end;

  DCLinkedHub.OnLinkedHubSearch := DCLNK_Search;
  DcLinkedHub.OnLinkedHubConnectToMe := DCLNK_ConnectToMe;
  DcLinkedHub.DCCMD_SendHeardbeats();

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Create] createing userlist');
  {$ENDIF}

  _HubUsers := TConnectedUserList.Create;

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Create] create finished');
  {$ENDIF}
end;

{ TDCHub Destroy procedure }
destructor TDCHub.Destroy;
begin
  inherited Destroy;
  try
  {$IFDEF DEBUG}
   writeln ('[TDCHub.Destroy] Destroy executed');
  {$ENDIF}


  {$IFDEF DEBUG}
   writeln ('[TDCHub.Destroy] Destroying LinkedHub socket');
  {$ENDIF}

  DCLinkedHub.Destroy;

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Destroy] Destroying userlist');
  {$ENDIF}
  _HubUsers.Destroy;

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Destroy] Destroying Server socket');
  {$ENDIF}

  { error on linux systems when users are connected i can't destroy the Listener }

  _HubTCPListener.Destroy;

  {$IFDEF DEBUG}
   writeln ('[TDCHub.Destroy] Destroy finished');
  {$ENDIF}
  except
  end;
end;

{ *** virtual propertys functions *** }

{ set the _HubTCPListener tcp socket activeflag}
procedure TDCHub._HubTCPListenerSetActive(Value: boolean);
begin
  { setting tcp server socket activeflag}
  { error on linux systems when users are connected i can't set the listerner to active := false }
  try
   _HubTCPListener.Active  := Value;
  except
   raise;
  end;
end;

{ get the _HubTCPListener tcp socket activeflag}
function TDCHub._HubTCPListenerGetActive():boolean; 
begin
 result := _HubTCPListener.Active;
end;

{ *** _HubTCPListener events *** }

{ this procedure will be executed when a user has disconnected }
procedure TDCHub.TCPConnectionClose(AThread: TIdPeerThread);
begin
 ODC_UserDisconnected(AThread);
end;

{ this procedure will be executed when a new user connected }
{ todo: nothing}
procedure TDCHub.TCPConnectionOpen(AThread: TIdPeerThread);
begin

 { waiting one second before handeling connection }
 { thats a little flood protection }
  sleep(1000);

  {$IFDEF DEBUG}
   writeln ('[TDCHub.TCPConnectionOpen] user connection opend');
  {$ENDIF}

  ODC_UserConnected(AThread);

end;

{ this procedure must executed it only calls the UserMessageLoop procedure }
{ todo: nothing }
procedure TDCHub.TCPExecute(AThread: TIdPeerThread);
var ArrayOffset:integer;
    tmp : string;
begin
while AThread.connection.connected = true do
begin
 sleep(1);
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(AThread);

 { own buffer managment begin }
 
 { DISABLED because there is a litte bug in the functions ;) 

 PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer := PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer + AThread.connection.CurrentReadBuffer;

 While pos('|',PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer) <> 0 do
 begin
      tmp := '';
      tmp := copy(PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer, 1, pos('|',PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer) -1);
      PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer := copy(PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer, Length(tmp) + 2,length(PConnectedUser(_HubUsers.Items[ArrayOffset])^.buffer));
      ODC_UserMessageParse(AThread,tmp);
 end;

 }

 { own buffer managment end }

 { indys buffer management begin }

 tmp := AThread.connection.ReadLn('|');
 if tmp <> '' then ODC_UserMessageParse(AThread,tmp);

 { indys buffer management end }

end;
{ when UserMessageLoop is finished, closing the client connection }
AThread.connection.Disconnect;
ODC_UserDisconnected(AThread);

end;

{ *** linked server control events ***}
                
{ this event will be executed, when a user is seraching on a linked hub}
procedure TDCHub.DCLNK_Search(SearchRequest:string);
begin
 if copy(SearchRequest,length(SearchRequest),1) = '|' then
 SearchRequest := copy(SearchRequest,1,length(SearchRequest) -1);
 DCCMD_SendBroadCastMsg(SearchRequest);
end;

{ this event will be executed, when a user is calling ConnectToMeStringon a linked hub}
procedure TDCHub.DCLNK_ConnectToMe(ConnectToMeString:string);
var Strings:stringarray;
begin

 Strings := splitstring(ConnectToMeString);
 
 {checking for crap}
 if length(Strings) <> 3 then
 begin
  { user sending crap }
 end
 {yea all was ok}
 else
 begin
  if (DCCMD_IsOnline(Strings[1]) = false) then
  begin
   { user not at this hub }
  end
  else
  begin
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(Strings[1]),'$ConnectToMe ' + Strings[1] + ' ' + Strings[2]);
  end;
 end;

end;

{ *** Increase / Decrease Function for TConnectedUserArray *** }

{ adding new user to the tconnecteduserarray }
function TDCHub.AddConnectedUser(socket:TIdPeerThread):boolean;
var tmpuser:TConnectedUser;
begin

 if (socket.connection.IOHandler as TIdIOHandlerSocket).Binding.PeerIP <> '' then
 begin
  tmpuser.Socket := socket;
  tmpuser.Username := 'unknown';
  tmpuser.RemoteIP := (socket.connection.IOHandler as TIdIOHandlerSocket).Binding.PeerIP;
  tmpuser.RemotePort := (socket.connection.IOHandler as TIdIOHandlerSocket).Binding.PeerPort;
  _HubUsers.Add(tmpuser);
  result := true;
 end
 else
 begin
  result := false;
 end;

end;

{ removing user from the tconnecteduserarray }
function TDCHub.DelConnectedUser(socket:TIdPeerThread):boolean;
begin
 _HubUsers.Delete(socket);
 result := true;
end;


{ *** Direct Connect Message Functions and Handeling *** }
{ this procedure calls a loop. if a new message from a client comes up,
  it parse the command and starts a other function to handle it. }
{ todo: nothing) }
procedure TDCHub.ODC_UserMessageParse(socket:TIdPeerThread;ReceivedData:string);
var ArrayOffset:integer;
    ReceivedCommand:string;
begin
    ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);

    if ReceivedData <> '' then
    begin
    { parsing starting }

    {$IFDEF DEBUG}
     writeln ('[TDCHub.ODC_UserMessageParse] Get: '+ ReceivedData);
    {$ENDIF}
     
    { checking if it is a command or a public chatmessage }
     if ReceivedData[1] = '$' then
     begin
      {is a command}
      ReceivedCommand := copy(ReceivedData,1,pos(' ',ReceivedData) - 1);
      if (ReceivedCommand = '') then ReceivedCommand := ReceivedData;
      {doing some security checks}

      {only allow the $Key command when the handshake has not done }
      if ((PConnectedUser(_HubUsers.Items[ArrayOffset])^.keyauthorisedone = false) and (PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin = false)) then
      begin
       if ReceivedCommand <> '$Key' then
       begin
        ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Sending no Key',1,0,0);
        PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
        {$IFDEF DEBUG}
        writeln ('[TDCHub.UserMessageLoop] Sending no Key');
        {$ENDIF}
       end;
      end;

      {only allow $ValidateNick and $MyPass message before user rights has been checked}
      if ((PConnectedUser(_HubUsers.Items[ArrayOffset])^.Passwordauthoriseinprogress = true) and (PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin = false)) then
      begin
       if (ReceivedCommand <> '$ValidateNick') then if (ReceivedCommand <> '$MyPass') then
       begin
        ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Sending no Nickname Validationdata',1,0,0);
        PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
        {$IFDEF DEBUG}
        writeln ('[TDCHub.UserMessageLoop] ValidateNick Faild');
        {$ENDIF}
       end;
      end;

      {starts the normal command parsing}

      if (PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin = false) then
      begin

       if ReceivedCommand = '$ValidateNick' then
       begin
        DCMSG_ValidateNick(socket,ReceivedData);
       end
       else if ReceivedCommand = '$MyPass' then
       begin
        DCMSG_MyPass(socket,ReceivedData);
       end
       else if ReceivedCommand = '$MyINFO' then
       begin
        DCMSG_MyINFO(socket,ReceivedData);
       end
       else if ReceivedCommand = '$GetNickList' then
       begin
        DCMSG_GetNickList(socket,ReceivedData);
       end
       else if ReceivedCommand = '$GetINFO' then
       begin
        DCMSG_GetINFO(socket,ReceivedData);
       end
       else if ReceivedCommand = '$To:' then
       begin
        DCMSG_To(socket,ReceivedData);
       end
       else if ReceivedCommand = '$Key' then
       begin
        DCMSG_Key(socket,ReceivedData);
       end
       else if ReceivedCommand = '$Version' then
       begin
        DCMSG_Version(socket,ReceivedData);
       end
       else if ReceivedCommand = '$Quit' then
       begin
        DCMSG_Quit(socket,ReceivedData);
       end
       else if ReceivedCommand = '$ConnectToMe' then
       begin
        DCMSG_ConnectToMe(socket,ReceivedData);
       end
       else if ReceivedCommand = '$MultiConnectToMe' then
       begin
        DCMSG_MultiConnectToMe(socket,ReceivedData);
       end
       else if ReceivedCommand = '$RevConnectToMe' then
       begin
        DCMSG_RevConnectToMe(socket,ReceivedData);
       end
       else if ReceivedCommand = '$SR' then
       begin
        DCMSG_SR(socket,ReceivedData);
       end
       else if ReceivedCommand = '$Search' then
       begin
        DCMSG_Search(socket,ReceivedData);
       end
       else if ReceivedCommand = '$MultiSearch' then
       begin
        DCMSG_MultiSearch(socket,ReceivedData);
       end
       else if ReceivedCommand = '$Kick' then
       begin
        DCMSG_Kick(socket,ReceivedData);
       end
       else if ReceivedCommand = '$OpForceMove' then
       begin
        DCMSG_OpForceMove(socket,ReceivedData);
       end
       else
       begin
        {command not found}
        {$IFDEF DEBUG}
         writeln ('[TDCHub.ODC_UserMessageParse] getting unknown command:' + ReceivedCommand);
        {$ENDIF}
       end;
     end;
     end
     else if ReceivedData[1] = '<' then
     begin
      if copy(ReceivedData,1,pos(' ',ReceivedData) - 1) = '<' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.username + '>' then
      begin       
       if copy(copy(ReceivedData,pos(' ',ReceivedData) + 1,length(ReceivedData)),1,1) = '%' then
       begin
        service(copy(copy(ReceivedData,pos(' ',ReceivedData) + 1,length(ReceivedData)),1,length(ReceivedData)),socket);
       end
       else
       begin
        ConsoleWriteln('[Chat] ' + ReceivedData,2,0,0);
        DCCMD_SendBroadCastMsg(ReceivedData);
       end;
      end
      else
      begin
       DCCMD_SendToUserMsg(socket,'<Hub-Security> You sending a bad source username.');
       PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
      end;
     end
     else
     begin
      { todo: disconnect user when this appears }
      {is somthing else}
     end;
    { parsing end }

    {kicking user when badlogin = true}

    if (PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin = true) then
    begin
     {$IFDEF DEBUG}
      writeln ('[TDCHub.ODC_UserMessageParse] closing socket (badlogin)');
     {$ENDIF}
     ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') TCP-Connection will be terminated',1,0,0);
     PConnectedUser(_HubUsers.Items[ArrayOffset])^.socket.connection.Disconnect;
     ODC_UserDisconnected(PConnectedUser(_HubUsers.Items[ArrayOffset])^.Socket);
    end;

    end;
end;

{ this procedure will called from the tcp onconnect event }
procedure TDCHub.ODC_UserConnected(socket:TIdPeerThread);
var ArrayOffset:integer;
begin

  { adding user to userlist }
  if (AddConnectedUser(socket) = true) then
  begin
   ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);

   ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') TCP-Connection opened',1,0,0);

   if IsBannedUser('',PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP) then
   begin
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Banned IP-Address ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP,1,0,0);
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Your IP-Address is banned!');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
   end
   else if _HubRedirectAllUsers then
   begin
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Redirecting all users to ' + _HubRedirectAddress,1,0,0);
    DCCMD_SendToUserMsg(socket,'<Hub-Security> You are being redirected to ' + _HubRedirectAddress + '.');
    DCCMD_SendToUserMsg(socket,'$ForceMove ' + _HubRedirectAddress);
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
   end
   else if (_HubUsers.count >= _HubMaxUsers) then
   begin
    if _HubRedirectIfHubIsFull then
    begin
     ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Hub is Full, redirecting all users to ' + _HubRedirectAddress,1,0,0);
     DCCMD_SendToUserMsg(socket,'<Hub-Security> This hub is currently full! You are being redirected to ' + _HubRedirectAddress + '.');
     DCCMD_SendToUserMsg(socket,'$ForceMove ' + _HubRedirectAddress);
     PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
    end
    else
    begin
     ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Hub is Full!',1,0,0);
     DCCMD_SendToUserMsg(socket,'<Hub-Security> This hub is currently full!');
     DCCMD_SendToUserMsg(socket,'$HubIsFull');
     PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
    end;
   end
   else
   begin
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.lock := createvalidlock;
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.key := createkey(PConnectedUser(_HubUsers.Items[ArrayOffset])^.lock,5);
    DCCMD_SendToUserMsg(socket,'$Lock ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.lock);
    DCCMD_SendToUserMsg(socket,'$HubName ' + _HubName);
    {$IFDEF USEDCWELCOME}
     DCCMD_SendToUserMsg(socket,'<Hub-Security> This hub is running version 1.0.25 of the NeoModus Direct Connect Hub Software.');
    {$ELSE}
     DCCMD_SendToUserMsg(socket,'<Hub-Security> This hub is running version ' + OpenDCdVersion + ' of the Open Direct Connect Server at ' + OSVersion);
    {$ENDIF}
   end;
   if (PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin = true) then
   begin
    sleep(1000);
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') TCP-Connection will be terminated',1,0,0);
    if PConnectedUser(_HubUsers.Items[ArrayOffset])^.Socket.connection.Connected = true then
    begin
     PConnectedUser(_HubUsers.Items[ArrayOffset])^.socket.connection.Disconnect;
     ODC_UserDisconnected(PConnectedUser(_HubUsers.Items[ArrayOffset])^.Socket);
    end;
   end;
  end
  else
  begin
   socket.connection.Disconnect;
   ODC_UserDisconnected(socket);
   ConsoleWriteln('[LOG] WARNING: Could not add user to Connected User list. TCP-Connection will be terminated',1,0,0);
  end;
end;

{ this procedure will executed from the tcp ondisconnect event }
{ and to disconnect a user too }
procedure TDCHub.ODC_UserDisconnected(socket:TIdPeerThread);
var username:string;
    LoggedIn:boolean;
    ArrayOffset:integer;
begin

 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);

 if ( ArrayOffset > -1 ) then
 begin

   {$IFDEF DEBUG}
    writeln ('[TDCHub.ODC_UserDisconnected] user connection closed');
   {$ENDIF}

   LoggedIn := PConnectedUser(_HubUsers.Items[ArrayOffset])^.SuccessLoggedIn;
   username := PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username;

   ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') TCP-Connection closed',1,0,0);

   { removing user from userlist }
   DelConnectedUser(socket);

   {broadcasting to everyone that client has left the hub}
   if LoggedIn = true then
   begin
    DCCMD_SendBroadCastMsg('$Quit ' + username);
   end;
  end;
end;


{ *** Direct Connect Message Parsing *** }

{ parsing the $ValidateNick message }
procedure TDCHub.DCMSG_ValidateNick(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 PConnectedUser(_HubUsers.Items[ArrayOffset])^.ValidateDenide := false;

 {check if nickname given}
 if length(Strings) = 1 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> You must insert a Nickname.');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {checking for other crap}
 else if length(Strings) <> 2 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $ValidateNick Command');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
   {check if nickname is banned}
   if IsBannedUser(Strings[1],'') then
   begin
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Your Nickname is banned!');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Banned Nickname ' + Strings[1],1,0,0);
   end
   {check if user is illegal}
   else if (IsIllegalUser(Strings[1]) = true) then
   begin
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Your Nickname not allowed.');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
   end
  {check if username allready taken}
   else if DCCMD_IsOnline(Strings[1]) = true then
   begin
    DCCMD_SendToUserMsg(socket,'$ValidateDenide ' + Strings[1]);
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.ValidateDenide := true;
   end
   else
   begin
    {check if user want reset his username}
    if PConnectedUser(_HubUsers.Items[ArrayOffset])^.SuccessLoggedIn = true then
    begin
     DCCMD_SendToUserMsg(socket,'<Hub-Security> You allready connected and authorised!');
     PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
    end
    else
    begin
     if (getusertype(Strings[1]) = 'E') then
     begin
      if (_HubIsRegisteredUsersOnly = true) then
      begin
        ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Non registered Nickname connected on a registered only Hub',1,0,0);
        DCCMD_SendToUserMsg(socket,'<Hub-Security> Only registered users allowed on this hub');
       PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
      end
      else
      begin
        PConnectedUser(_HubUsers.Items[ArrayOffset])^.Passwordauthoriseinprogress := false;
        PConnectedUser(_HubUsers.Items[ArrayOffset])^.SuccessLoggedIn := true;
        PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username := Strings[1];
        DCCMD_SendBroadCastMsg('$Hello ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username);
        ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') User connected.',1,0,0);
        informuseraboutmemos(socket);
        DCCMD_SendMOTD(socket);
      end;
     end
     else
     begin
      PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username := Strings[1];
      DCCMD_SendToUserMsg(socket,'<Hub-Security> You are using a password protected nickname, please enter the password.');
      ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Password protected Nickname, waiting for Password',1,0,0);
      DCCMD_SendToUserMsg(socket,'$GetPass'); //for Passwd protected Nicks
     end;
    end;
   end;
 end;
end;

{ parsing the $MyPass message }
procedure TDCHub.DCMSG_MyPass(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);

 {checking for crap}
 if length(Strings) <> 2 then
 begin
  ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Password protected Nickname, receiving no Password',1,0,0);
  DCCMD_SendToUserMsg(socket,'<Hub-Security> You have insert no Password. Good bye!');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
  {validate password}
  if (validateuser(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username,Strings[1]) <> 1) then
  begin
   ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Password protected Nickname, receiving bad Password',1,0,0);
   DCCMD_SendToUserMsg(socket,'<Hub-Security> You have insert a wrong Password. Good bye!');
   DCCMD_SendToUserMsg(socket,'$BadPass');
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true; 
  end
  else
  begin
   {check user rights}
   if (getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'U') then
   begin
    {user is a normal user}
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Password protected Nickname, User sucessfully connected',1,0,0);
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Password correct! Welcome at this Hub!');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.Passwordauthoriseinprogress := false;
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.SuccessLoggedIn := true;
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.Password := Strings[1];
    DCCMD_SendBroadCastMsg('$Hello ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username);
    informuseraboutmemos(socket);
    DCCMD_SendMOTD(socket);
   end
   else if ((getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'O') or (getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'M')) then
   begin
    ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Password protected Nickname, Operator successfully connected',1,0,0);
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Password correct! Welcome at this hub!');
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Logged in.');
    DCCMD_SendToUserMsg(socket,'$LogedIn ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username);
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.Passwordauthoriseinprogress := false;
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.SuccessLoggedIn := true;
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.Password := Strings[1];
    DCCMD_SendBroadCastMsg('$Hello ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username);
    DCCMD_SendOperatorlist();
    informuseraboutmemos(socket);
    DCCMD_SendMOTD(socket);
   end;
  end;
 end;
end;

{ parsing the $MyINFO message }
procedure TDCHub.DCMSG_MyINFO(socket:TIdPeerThread;msg:string);
var ArrayOffset:integer;
    tmpnfo1:string;
    tmpnfo2:string;
    tmpnick:string;
    tmpspeed:string;
    tmpbytes:string;
    tmpcomment:string;
    tmpmail:string;
    tmpstat:string;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);

 tmpnfo1 := Copy(msg,9,length(msg));
 tmpnfo2 := copy(tmpnfo1, 1, pos(' ',tmpnfo1) -1);
 tmpnfo1 := copy(tmpnfo1, pos(' ',tmpnfo1) + 1,length(tmpnfo1));
 tmpnick := copy(tmpnfo1, 1, pos(' ',tmpnfo1) -1);
 tmpnfo1 := copy(tmpnfo1, pos(' ',tmpnfo1) + 1,length(tmpnfo1));
 tmpcomment := copy(tmpnfo1, 1, pos('$ $',tmpnfo1) -1);
 tmpnfo1 := copy(tmpnfo1, pos('$ $',tmpnfo1) + 3,length(tmpnfo1));
 tmpspeed := copy(tmpnfo1, 1, pos('$',tmpnfo1) -2);
 tmpstat := copy(tmpnfo1, pos('$',tmpnfo1) - 1, 1);
 tmpnfo1 := copy(tmpnfo1, pos('$',tmpnfo1) + 1,length(tmpnfo1));
 tmpmail := copy(tmpnfo1, 1, pos('$',tmpnfo1) -1);
 tmpnfo1 := copy(tmpnfo1, pos('$',tmpnfo1) + 1,length(tmpnfo1));
 tmpbytes := copy(tmpnfo1, 1, pos('$',tmpnfo1) -1);

 if tmpnick = PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username then
  begin
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.Speed := tmpspeed;
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.Sharedbytes := strtoint64(tmpbytes);
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.Description := tmpcomment;
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.Email := tmpmail;
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.Status := tmpstat[1];

  if (_HubMinFileShare > PConnectedUser(_HubUsers.Items[ArrayOffset])^.Sharedbytes) then
  begin
    DCCMD_SendToUserMsg(socket,'<Hub-Security> You dont meet the minimum share of ' + inttostr(_HubMinFileShare) + ' bytes.');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
  end
  else
  begin
     DCCMD_SendBroadCastMsg('$MyINFO $ALL ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Description + '$ $' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Speed + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Status  + '$' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Email + '$' + inttostr(PConnectedUser(_HubUsers.Items[ArrayOffset])^.Sharedbytes) + '$');
  end;
 end
 else
 begin
    DCCMD_SendToUserMsg(socket,'<Hub-Security> Bad MyINFO command! You only could change your own Informations');
    PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end;
end;

{ parsing $GetNicklist message }
procedure TDCHub.DCMSG_GetNickList(socket:TIdPeerThread;msg:string);
begin
 DCCMD_SendNicklist(socket);
 DCCMD_SendOperatorlist(socket);
end;

{ parsing $GetINFO message }
procedure TDCHub.DCMSG_GetINFO(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
    n:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);

 {checking for crap}
 if length(Strings) <> 3 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $GetINFO Command');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
  for n := 0 to _HubUsers.Count - 1 do
  begin

     if PConnectedUser(_HubUsers.Items[n])^.username = Strings[1] then DCCMD_SendToUserMsg(socket,'$MyINFO $ALL ' + PConnectedUser(_HubUsers.Items[n])^.Username + ' ' + PConnectedUser(_HubUsers.Items[n])^.Description + '$ $' + PConnectedUser(_HubUsers.Items[n])^.Speed + PConnectedUser(_HubUsers.Items[n])^.Status + '$' + PConnectedUser(_HubUsers.Items[n])^.Email + '$' + inttostr(PConnectedUser(_HubUsers.Items[n])^.Sharedbytes) + '$|');
  end;
 end;
end;

{ parsing $To: message }
procedure TDCHub.DCMSG_To(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
    TmpString:string;
    i : integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 
 {checking for crap}
 if length(Strings) < 6 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $To: Command');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
  if (Strings[3] <> PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $To: Command. You only could send messages with your name.');
   PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
  end
  else if (DCCMD_IsOnline(Strings[1]) = false) then
  begin
   DCCMD_SendToUserMsg(socket,'$To: ' + Strings[3] + ' From: ' + Strings[1] + ' $<Hub-Security> Sorry, the User "' + Strings[1] + '" is not online');
  end
  else
  begin
   for i := 5 to length(Strings) - 1 do TmpString := TmpString + ' ' + Strings[i]; 
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(Strings[1]),'$To: ' + Strings[1] + ' From: ' + Strings[3] + ' $<' + Strings[3] + '>' + TmpString);
  end;
 end;
end;

{ parsing the $Key message }
procedure TDCHub.DCMSG_Key(socket:TIdPeerThread;msg:string);
var KeyString:string;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 KeyString := copy(msg,6,length(msg));

 if (PConnectedUser(_HubUsers.Items[ArrayOffset])^.key <> KeyString) then
 begin
  {key was bad}
  ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') Sending a bad Key',1,0,0);
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 else
 begin
 {key was ok}
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.keyauthorisedone := true;
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.Passwordauthoriseinprogress := true;
 end;
end;

{ parsing the $Version message }
procedure TDCHub.DCMSG_Version(socket:TIdPeerThread;msg:string);
begin
 { don't parse it }
end;

{ parsing the $Quit message }
procedure TDCHub.DCMSG_Quit(socket:TIdPeerThread;msg:string);
begin
 { don't parse it }
end;


{ parsing the $ConnectToMe message }
procedure TDCHub.DCMSG_ConnectToMe(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 
 {checking for crap}
 if length(Strings) <> 3 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $ConnectToMe: Command');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
  if (DCCMD_IsOnline(Strings[1]) = false) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sorry, the User "' + Strings[1] + '" is not online');
  end
  else
  begin
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(Strings[1]),'$ConnectToMe ' + Strings[1] + ' ' + Strings[2]);
  end;
 end;
end;

{ parsing the $MultiConnectToMe message }
procedure TDCHub.DCMSG_MultiConnectToMe(socket:TIdPeerThread;msg:string);
begin
 DcLinkedHub.DCCMD_MultiConnectToMe(msg);
end;

{ parsing the $RevConnectToMe message }
procedure TDCHub.DCMSG_RevConnectToMe(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 
 {checking for crap}
 if length(Strings) <> 3 then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> Sending bad $RevConnectToMe: Command');
  PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 end
 {yea all was ok}
 else
 begin
  if (DCCMD_IsOnline(Strings[1]) = false) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sorry, the User "' + Strings[1] + '" is not online');
  end
  else
  begin
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(Strings[1]),'$RevConnectToMe ' + Strings[1] + ' ' + Strings[2]);
  end;  
 end;
end;

{ parsing the $SR message }
procedure TDCHub.DCMSG_SR(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 if (_HubDisablePassiveSearch = false) then
 begin
  ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
  Strings := splitstring(msg,#5);

  {checking for crap}
  if length(Strings) = 4 then
  begin
   if (DCCMD_IsOnline(Strings[3]) = true) then
   begin
    DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(Strings[3]),Strings[0] + #5 + Strings[1] + #5 + Strings[2]);
   end
  end;
 end;
end;

{ parsing the $Search message }
procedure TDCHub.DCMSG_Search(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
// _HubDisablePassiveSearch
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 if (length(Strings[1]) > 3) then
 begin
  // checks if passive search
  if ((uppercase(copy(Strings[1],1,4)) = 'HUB:') and _HubDisablePassiveSearch = true) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sorry passive searching is disabled at this Hub. Please switch to active mode.');
  end
  //is active search
  else
  begin
   DCCMD_SendBroadCastMsg(msg);
  end;
 end;
end;

{ parsing the $MultiSearch message }
procedure TDCHub.DCMSG_MultiSearch(socket:TIdPeerThread;msg:string);
begin
 DcLinkedHub.DCCMD_MultiSearch(msg);
end;

{ parsing the $Kick message }
procedure TDCHub.DCMSG_Kick(socket:TIdPeerThread;msg:string);
var Strings:stringarray;
    ArrayOffset:integer;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 Strings := splitstring(msg);
 
 if ((getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'O') or (getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'M'))  then
 begin
  if (DCCMD_IsOnline(Strings[1]) = false) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sorry, the User "' + Strings[1] + '" is not online');
  end
  else
  begin
   DCCMD_KickUser(DCCMD_GetSocketbyUsername(Strings[1]));
   DCCMD_SendToUserMsg(socket,'<Hub-Security> The User "' + Strings[1] + '" has been kicked!');
  end;
 end
 else
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> You have no permissions to kick other users!');
 end;
end;

{ parsing the $OpForceMove message }
procedure TDCHub.DCMSG_OpForceMove(socket:TIdPeerThread;msg:string);
var ArrayOffset:integer;
    tmpnfo1 :string;
    tmpnick :string;
    tmplocation :string;
    tmpmsg :string;
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);

 tmpnfo1 := copy(msg,pos(' ',msg) + 1,length(msg));
 tmpnick := copy(tmpnfo1,6,pos('$Where',tmpnfo1)-6);
 tmpnfo1 := copy(tmpnfo1,pos('$Where',tmpnfo1) + 1,length(tmpnfo1));
 tmplocation := copy(tmpnfo1,7,pos('$Msg',tmpnfo1)-7);
 tmpmsg := copy(tmpnfo1,pos('$Msg',tmpnfo1) + 5,length(tmpnfo1));
 
 if ((getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'O') or (getusertype(PConnectedUser(_HubUsers.Items[ArrayOffset])^.username) = 'M'))  then
 begin
  if (DCCMD_IsOnline(tmpnick) = false) then
  begin
   DCCMD_SendToUserMsg(socket,'<Hub-Security> Sorry, the User "' + tmpnick + '" is not online');
  end
  else
  begin
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(tmpnick),'<Hub-Security> ' + tmpmsg);
   DCCMD_SendToUserMsg(DCCMD_GetSocketbyUsername(tmpnick),'$ForceMove ' + tmplocation);
   DCCMD_SendToUserMsg(socket,'<Hub-Security> The User "' + tmpnick + '" has been redirected!');

   PConnectedUser(_HubUsers.Items[DCCMD_GetArrayOffsetbySocket(DCCMD_GetSocketbyUsername(tmpnick))])^.BadLogin := true;
  end;
 end
 else
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> You have no permissions to redirect other users!');
 end;

end;

{ *** Direct Connect Functions *** }

{ sending to each connected user that successfully loged in a message }
function TDCHub.DCCMD_SendBroadCastMsg(msg:string):boolean;
var n:integer;
begin
for n := 0 to (_HubUsers.count - 1) do
begin
 if PConnectedUser(_HubUsers.Items[n])^.SuccessLoggedIn = true then
 begin
  DCCMD_SendToUserMsg(PConnectedUser(_HubUsers.Items[n])^.socket,msg);
 end;
end;
result := true;
end;

{ redirecting all users to an other hub }
function TDCHub.DCCMD_RedirectAllUsers():boolean;
var n:integer;
begin
for n := 0 to (_HubUsers.count - 1) do
begin
 if PConnectedUser(_HubUsers.Items[n])^.SuccessLoggedIn = true then
 begin
  DCCMD_SendToUserMsg(PConnectedUser(_HubUsers.Items[n])^.socket,'<Hub-Security> You are being redirected to ' + _HubRedirectAddress + '.');
  DCCMD_SendToUserMsg(PConnectedUser(_HubUsers.Items[n])^.socket,'$ForceMove ' + _HubRedirectAddress);
  PConnectedUser(_HubUsers.Items[n])^.BadLogin := true;
 end;
end;
result := true;
end;

{ kicking all users }
function TDCHub.DCCMD_KickAllUsers():boolean;
var n:integer;
begin
for n := 0 to (_HubUsers.count - 1) do
begin
 PConnectedUser(_HubUsers.Items[n])^.BadLogin := true;
 DCCMD_KickUser(PConnectedUser(_HubUsers.Items[n])^.Socket);
end;
end;

{ sending to spezific user a message }
function TDCHub.DCCMD_SendToUserMsg(socket:TIdPeerThread;msg:string):boolean;
var done : boolean;
begin
done := false;
if socket <> nil then
begin
 if (Socket.connection.Connected = true) then
 begin
  Socket.connection.Write(msg + '|');
  done := true;
 end;
end;
result := done;
end;

{ returns the socket of a spezific username }
function TDCHub.DCCMD_GetSocketbyUsername(username:smallstring): TIdPeerThread;
var n:integer;
    tmpsocket:TIdPeerThread;
begin
tmpsocket := nil;
for n := 0 to _HubUsers.count - 1 do
begin
 if uppercase(PConnectedUser(_HubUsers.Items[n])^.username) = uppercase(username) then
 begin
  tmpsocket := PConnectedUser(_HubUsers.Items[n])^.socket;
  break;
 end;
end;
result := tmpsocket;
end;

{ returns username }
function TDCHub.DCCMD_GetUsernamebySocket(socket:TIdPeerThread):smallstring;
var n:integer;
    tmpuser:smallstring;
begin
tmpuser := '';
if socket <> nil then
begin
 for n := 0 to _HubUsers.Count - 1 do
 begin
  if PConnectedUser(_HubUsers.Items[n])^.socket = socket then
  begin
   tmpuser := PConnectedUser(_HubUsers.Items[n])^.username;
   break;
  end;
 end;
end;
result := tmpuser;
end;

{ returns the id of a spezific socket }
function TDCHub.DCCMD_GetArrayOffsetbySocket(socket:TIdPeerThread): integer;
var n:integer;
    tmpid:integer;
begin
tmpid := -1;
if socket <> nil then
begin
 for n := 0 to _HubUsers.count - 1 do
 begin
  if PConnectedUser(_HubUsers.Items[n])^.socket = socket then
  begin
   tmpid := n;
   break;
  end;
 end;
end;
result := tmpid;
end;

{ checks if  a spezific user is online }
function TDCHub.DCCMD_IsOnline(username:string):boolean;
var n:integer;
    tmpbol:boolean;
begin
 tmpbol := false;
   for n := 0 to _HubUsers.count - 1 do
   begin
     if (uppercase(PConnectedUser(_HubUsers.Items[n])^.username) = uppercase(username)) then
     begin
       tmpbol := true;
     end;
   end;
 result := tmpbol;
end;

{ sends nicklist to a spezific user }
procedure TDCHub.DCCMD_SendNicklist(socket:TIdPeerThread = nil);
var x : string;
    n : integer;
begin
if socket <> nil then
begin
   x := '$NickList ';
   for n := 0 to _HubUsers.count - 1 do
   begin
    if PConnectedUser(_HubUsers.Items[n])^.SuccessLoggedIn = true then x := x + PConnectedUser(_HubUsers.Items[n])^.username + '$$';
   end;
   if socket <> nil then DCCMD_SendToUserMsg(socket,x)
   else DCCMD_SendBroadCastMsg(x);
end;
end;

{ sends oplist to a spezific user }
procedure TDCHub.DCCMD_SendOperatorlist(socket:TIdPeerThread = nil);
var x : string;
    n : integer;
begin
if socket <> nil then
begin
   x := '$OpList ';
   for n := 0 to _HubUsers.count - 1 do
   begin
    if (((getusertype(PConnectedUser(_HubUsers.Items[n])^.username) = 'O') or (getusertype(PConnectedUser(_HubUsers.Items[n])^.username) = 'M')) and (PConnectedUser(_HubUsers.Items[n])^.SuccessLoggedIn = true)) then x := x + PConnectedUser(_HubUsers.Items[n])^.username + '$$';
   end;
   if socket <> nil then DCCMD_SendToUserMsg(socket,x)
   else DCCMD_SendBroadCastMsg(x);
end;
end;

{ kicks the spezific user with a reason }
function TDCHub.DCCMD_KickUser(socket:TIdPeerThread;msg:string = ''):boolean;
var ArrayOffset:integer;
begin
if (socket = nil) then
begin
 result := false;
end
else
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') connection will be terminated (by kick command)',1,0,0);
 if msg <> '' then
 begin
  DCCMD_SendToUserMsg(socket,'<Hub-Security> You will be kicked, reason : "' + msg + '"');
  sleep(1000);
 end;
 try
  socket.connection.Disconnect;
 except
 end;
 ODC_UserDisconnected(socket);
 result := true;
end;
end;

{ redirect the spezific user with a reason }
function TDCHub.DCCMD_RedirectUser(socket:TIdPeerThread):boolean;
var ArrayOffset:integer;
begin
if (socket = nil) then
begin
 result := false;
end
else
begin
 ArrayOffset := DCCMD_GetArrayOffsetbySocket(socket);
 ConsoleWriteln('[LOG] ' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.Username + ' (' + PConnectedUser(_HubUsers.Items[ArrayOffset])^.RemoteIP + ') will be redirected',1,0,0);
 DCCMD_SendToUserMsg(socket,'<Hub-Security> You are being redirected to ' + _HubRedirectAddress + '.');
 DCCMD_SendToUserMsg(socket,'$ForceMove ' + _HubRedirectAddress);
 PConnectedUser(_HubUsers.Items[ArrayOffset])^.BadLogin := true;
 result := true;
end;
end;

{ sends to a spezific User the MOTD }
procedure TDCHub.DCCMD_SendMOTD(socket:TIdPeerThread);
var  motdfile: TextFile;
     tmp:string;
begin
if (socket <> nil) then
begin
 if fileexists('motd') <> false then
 begin
  AssignFile(motdfile, 'motd');
  Reset(motdfile);
  while not Eof(motdfile) do
  begin
   Readln(motdfile,tmp);
   DCCMD_SendToUserMsg(socket,'<MOTD> ' + tmp);
  end;
  CloseFile(motdfile);
 end;
end;
end;

end.
 