{ Copyright (C) 2001 suxxx@users.sourceforge.net
This is free software distributed under the terms of the
GNU Public License.  See the file COPYING for details.
$Id: dc_server_server.pas,v 0.1 2002/01/03 16:30 suxxx $
}

{ here all server to server interactions stored }

{ to use debug messages }
//{$DEFINE DEBUG}

unit dc_server_server;

interface

uses
     { kylix / delphi units }
     SysUtils,
     classes,
     { indy units }
     IdIPWatch,
     IdBaseComponent,
     IdComponent,
     IdSocketHandle,
     IdUDPBase,
     IdUDPServer,
     IdGlobal,
     { opendcd units }
     declare_types,
     declare_vars,
     threadtimer,
     global_linkedserver_functions,
     global_multiconsol_functions,
     global_functions;

type TDCLinkedHubSearchEvent = procedure(SearchRequest:string) of object;
type TDCLinkedHubConnectToMeEvent = procedure(ConnectToMeString:string) of object;

type
  TDCLinkedHub = class(TComponent)
  protected
  _DCLinkedServerListener : TIdUDPServer;
  _DCLinkedServers:TLinkedServerArray;
  _DCLinkedHubSearchEvent:TDCLinkedHubSearchEvent;
  _DCLinkedHubConnectToMeEvent:TDCLinkedHubConnectToMeEvent;
  _DCLinkedServerTimer:TThreadTimer;
  {none}
  private
  { _DCLinkedServerListener events }
   procedure UDPReceivingData(Sender: TObject; AData: TStream; ABinding: TIdSocketHandle);
  { DC Message Handeling }
   procedure DCMSG_Up(Data:string;ABinding: TIdSocketHandle);
   procedure DCMSG_UpToo(Data:string;ABinding: TIdSocketHandle);
   procedure DCMSG_Search(Data:string;ABinding: TIdSocketHandle);
   procedure DCMSG_ConnectToMe(Data:string;ABinding: TIdSocketHandle);
  public
   { functions / procedure declarations }
   constructor Create(ipaddress:string;port:integer);
   destructor Destroy();

   procedure DCCMD_MultiSearch(SearchCMD:string);
   procedure DCCMD_MultiConnectToMe(ConnectToMeCMD:string);
   procedure DCCMD_SendHeardbeats();
   {properties}
   property OnLinkedHubSearch: TDCLinkedHubSearchEvent read _DCLinkedHubSearchEvent write _DCLinkedHubSearchEvent;
   property OnLinkedHubConnectToMe: TDCLinkedHubConnectToMeEvent read _DCLinkedHubConnectToMeEvent write _DCLinkedHubConnectToMeEvent;
  end;
implementation


{ TDCLinkedHub constructor }
constructor TDCLinkedHub.Create(ipaddress:string;port:integer);
var socket : TIdSocketHandle;
begin
  inherited Create(nil);

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Create] Create executed');
  {$ENDIF}

  { creating the udp server socket }

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Create] Createting udp socket');
  {$ENDIF}  
  _DCLinkedServerListener := TIdUDPServer.Create(nil);
  socket := _DCLinkedServerListener.Bindings.Add;
  socket.IP := ipaddress;
  socket.Port := port;

  { enabling threaded events }
  _DCLinkedServerListener.ThreadedEvent := true;

  { linking Udp Events with local procedures }
  _DCLinkedServerListener.OnUDPRead := UDPReceivingData;

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Create] activating udp socket');
  {$ENDIF}

  { activating udp server socket }
  try
   _DCLinkedServerListener.Active := true;
  except
   {none}
  end;

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Create] creating heardbeattimer');
  {$ENDIF}

  { creating headbeat timer for linekd servers }
              
  _DCLinkedServerTimer := TThreadTimer.Create(60);
  _DCLinkedServerTimer.OnTimer := DCCMD_SendHeardbeats;

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Create] create finished');
  {$ENDIF}
end;

{ TDCLinkedHub Destroy procedure }
destructor TDCLinkedHub.Destroy;
begin
  inherited Destroy;

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Destroy] Destroy executed');
  {$ENDIF}

  _DCLinkedServerTimer.Terminate;
  _DCLinkedServerTimer.Destroy;
  try
   _DCLinkedServerListener.Active := false;
  except
   {none}
  end;
  _DCLinkedServerListener.Destroy;

  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.Destroy] Destroy finished');
  {$ENDIF}
end;

{ this event procedure will be executed when new udp data received }
procedure TDCLinkedHub.UDPReceivingData(Sender: TObject; AData: TStream; ABinding: TIdSocketHandle);
var ReceivedData:string;
    ReceivedLength:integer;
    ReceivedCommand:string;
    buf:^largebuf;
begin
  {$IFDEF DEBUG}
   writeln ('[TDCLinkedHub.UDPReceivingData] Receiving Data');
  {$ENDIF}

 ReceivedLength := AData.Size;

 Getmem(buf,ReceivedLength);
 setlength(ReceivedData,ReceivedLength);
 AData.Read(buf^,ReceivedLength);
 move(buf^,ReceivedData[1],ReceivedLength);
 freemem(buf);

 if ReceivedData <> '' then
 begin
 { parsing starting }

  { checking if it is a command or crap }
  if ReceivedData[1] = '$' then
  begin
   {is a command}
   ReceivedCommand := copy(ReceivedData,1,pos(' ',ReceivedData) - 1);

   if ReceivedCommand = '$Up' then
   begin
    DCMSG_Up(ReceivedData,ABinding);
   end
   else if ReceivedCommand = '$UpToo' then
   begin
    DCMSG_UpToo(ReceivedData,ABinding);
   end
   else if ReceivedCommand = '$Search' then
   begin
    DCMSG_Search(ReceivedData,ABinding);
   end
   else if ReceivedCommand = '$ConnectToMe' then
   begin
    DCMSG_ConnectToMe(ReceivedData,ABinding);
   end
   else
   begin
    { unknown command }
   end;
  end;
 end;
end;

{ *** UDP message parsing ***}

{ parsing the $Up message }
procedure TDCLinkedHub.DCMSG_Up(Data:string;ABinding: TIdSocketHandle);
var ServerId:integer;
    Strings:stringarray;
    CurIP : TIdIPWatch;
begin
  CurIP := TIdIPWatch.Create(nil);

  Strings := splitstring(Data);

  ServerId := IsAllowedLinkedServer(Abinding.PeerIP,aBinding.PeerPort,Strings[1]);
  if ServerId > -1 then
  begin
    ConsoleWriteln('[UDP] Receiving Heardbeat request. (' + Abinding.PeerIP + ':' + inttostr(Abinding.PeerPort) + ')',1,0,0);
   try
    ConsoleWriteln('[UDP] Sending Heardbeat response (' + Abinding.PeerIP + ':' + inttostr(Abinding.PeerPort) + ')',1,0,0);
    _DCLinkedServerListener.send(Abinding.PeerIP,aBinding.PeerPort,'$UpToo ' + linkedservers[ServerId].HubPassword + ' ' + CurIP.LocalIP + '|');
   except
    {none}
   end;
  end
  else
  begin
   ConsoleWriteln('[UDP] Receiving illegal Heardbeat request. (' + Abinding.PeerIP + ':' + inttostr(Abinding.PeerPort) + ')',1,0,0);
  end;
  CurIp.Free;
end;


{ parsing the $UpToo message }
procedure TDCLinkedHub.DCMSG_UpToo(Data:string;ABinding: TIdSocketHandle);
var ServerId:integer;
    Strings:stringarray;
begin
  Strings := splitstring(Data);

  ServerId := IsAllowedLinkedServer(Abinding.PeerIP,aBinding.PeerPort,Strings[1]);
  if ServerId > -1 then
  begin
    SetLinkedServerActive(Abinding.PeerIP,aBinding.PeerPort,Strings[1]);
    ConsoleWriteln('[UDP] Receiving Heardbeat response. (' + Abinding.PeerIP + ':' + inttostr(Abinding.PeerPort) + ')',1,0,0);
   end
  else
  begin
   ConsoleWriteln('[UDP] Receiving illegal Heardbeat response. (' + Abinding.PeerIP + ':' + inttostr(Abinding.PeerPort) + ')',1,0,0);
  end;
end;

{ parsing the $Search message }
procedure TDCLinkedHub.DCMSG_Search(Data:string;ABinding: TIdSocketHandle);
var ServerId:integer;
begin
  ServerId := IsOnlineLinkedServer(Abinding.PeerIP,aBinding.PeerPort);
  if ServerId > -1 then
  begin
   { fires the OnLinkedHubSearch Event }
    OnLinkedHubSearch(Data);
  end
  else
  begin
   ConsoleWriteln('[UDP] Receiving illegal search message',1,0,0);
  end;
end;

{ parsing the $ConnectToMe message }
procedure TDCLinkedHub.DCMSG_ConnectToMe(Data:string;ABinding: TIdSocketHandle);
var ServerId:integer;
begin
  ServerId := IsOnlineLinkedServer(Abinding.PeerIP,aBinding.PeerPort);
  if ServerId > -1 then
  begin
   { fires the OnLinkedHubConnectToMe Event }
   OnLinkedHubConnectToMe(Data);
  end
  else
  begin
   ConsoleWriteln('[UDP] Receiving illegal connecttome message',1,0,0);
  end;
end;

{ *** public proecedures *** }

{ sending to all linked severs}
procedure TDCLinkedHub.DCCMD_MultiSearch(SearchCMD:string);
var Strings:stringarray;
    i:integer;
begin
 Strings := splitstring(SearchCMD);
 if (length(Strings) = 3) then
 begin
   for i := 0 to (length(linkedservers) -1) do
   begin
    if linkedservers[i].HubStatus = 2 then
    begin
     _DCLinkedServerListener.Send(linkedservers[i].HubIP,linkedservers[i].HubPort,'$Search ' + Strings[1] + ' ' + Strings[2] + '|');
    end;
   end;
 end;
end;

{ sending only to linked server that in the $MultiConnectToMe message }
procedure TDCLinkedHub.DCCMD_MultiConnectToMe(ConnectToMeCMD:string);
var Strings:stringarray;
    i:integer;
    TmpIp : string;
begin
 Strings := splitstring(ConnectToMeCMD);
 if (length(Strings) = 4) then
 begin
   for i := 0 to (length(linkedservers) -1) do
   begin
    TmpIp := linkedservers[i].HubIP;
    if linkedservers[i].HubPort <> 411 then TmpIp := TmpIp + ':' + inttostr(linkedservers[i].HubPort);
    if ((linkedservers[i].HubStatus = 2) and (TmpIp = Strings[3])) then
    begin                                                        
     _DCLinkedServerListener.Send(linkedservers[i].HubIP,linkedservers[i].HubPort,'$ConnectToMe ' + Strings[1] + ' ' + Strings[2] + '|');
    end;
   end;
 end;
end;

{ sending to each client a server heardbeat }
procedure TDCLinkedHub.DCCMD_SendHeardbeats();
var i:integer;
    CurIP : TIdIPWatch;
begin
CurIP := TIdIPWatch.Create(nil);
for i := 0 to (length(linkedservers) -1) do
begin
 if linkedservers[i].HubStatus = 1 then
 begin
  linkedservers[i].HubStatus := 3;
 end
 else
 begin
  linkedservers[i].HubStatus := 1;
 end;
 ConsoleWriteln('[UDP] Sending Heardbeat request to ' + linkedservers[i].HubIP,1,0,0);
 try
  _DCLinkedServerListener.send(linkedservers[i].HubIP,linkedservers[i].HubPort,'$Up ' + linkedservers[i].HubPassword + ' ' + CurIP.LocalIP + '|');
 except
 end;
end;
CurIp.Free;
end;

end.
