{ Copyright (C) 2001 suxxx@users.sourceforge.net
This is free software distributed under the terms of the
GNU Public License.  See the file COPYING for details.
$Id: declare_types.pas,v 0.1 2002/01/03 16:30 suxxx $
}
   
{ here all custom types stored }

unit declare_types;

interface
uses
     { kylix / delphi units}
     Classes,
     SysUtils,
     { indy units }
     IdTCPServer;

{ some special variable types }
type largebuf=array[0..16777215] of byte;
type smallbuf=array[0..65535] of char;
type stringarray=array of string[255];
type littlestring=string[10];
type smallstring=string[20];
type middlestring=string[50];
type bigstring=string[100];
type xtrastring=string[200];
{ user type declaration for registered users }
type
  TRegUsers = record
        Username:smallstring;
        Password:littlestring;
        UserType:char;  {E = Error, U = User, O = Operator, M = Master }
  end;


{ currently connected user type declaration }
type
  TConnectedUser = record
        { users settings }
        Username:smallstring;
        Password:littlestring;
        RemoteIP:middlestring;
        Remoteport:integer;
        Speed:middlestring;
        Sharedbytes : int64;
        Description : bigstring;
        Email : bigstring;
        Status :char;
        { tcp socket from user }
        //Socket: TIdTCPServerConnection;
        Socket:TIdPeerThread;
        { some internal variables }
        SuccessLoggedIn:boolean;              {login progress was done}
        BadLogin :boolean;                    {needed for login progress}
        ValidateDenide :boolean;              {needed for login progress}
        Passwordauthoriseinprogress :boolean; {needed for login progress}
        lock : string[200];                   {needed for lock/key handshake}
        key: string[200];                     {needed for lock/key handshake}
        keyauthorisedone :boolean;            {key handshake done}
        buffer : string;                      {tcp receive buffer}
  end;
type PConnectedUser = ^TConnectedUser;

type TConnectedUserList = class(TList)
  function Add(Value: TConnectedUser): PConnectedUser;
  procedure Clear; override;
  procedure Delete(socket:TIdPeerThread); overload;
  procedure Delete(Index: Integer); overload;
  constructor Create;
  destructor Destroy; override;
end;

{hub settings}
type
  THubSettings = record
        HubName:smallstring;
        HubDescription:bigstring;
        HubAddress:middlestring;
        HubPort:integer;
        HubMaxUsers:integer;
        HubMinFileShare:int64;
        HubMinShareSlots:char;
        HubRedirectAddress:middlestring;
        HubRedirectAllUsers:boolean;
        HubRedirectIfHubIsFull:boolean;
        HubIsRegisteredUsersOnly:boolean;
        HubRegisteratPublic:boolean;
        HubDisablePassiveSearch:boolean;
  end;

{multiconsol type}
type
  TConsoleScreen = record
        CommandScreenBuf : array[0..24] of string[120];
        LoggingScreenBuf : array[0..24] of string[120];
        ChatScreenBuf : array[0..24] of string[120];
        LoggingScreenEnabled : boolean;
        ChatScreenEnabled : boolean;
        CommandScreenTmpBuffer : string[120];
        LoggingScreenTmpBuffer : string[120];
        ChatScreenTmpBuffer : string[120];
end;

{ linked servers type }
type
  TLinkedServer = record
        HubIP:string[15];
        HubPort:integer;
        HubPassword:smallstring;
        HubStatus:Integer;    //1 = unknown, 2 = online, 3 = offline
end;


type TLinkedServerArray = array of TLinkedServer;

{ banned user type }
type
  TBannedUser = record
        Username : string[20];
        IPAddress : string[15];
        BannedTo : TDateTime;
end;

{ memo type }

type
  TMemoServMessages = record
        FromUser:string[20];
        ToUser:string[20];
        TextMessage:string[160];
        SendDate:string[20];
        Readed:boolean;
  end;


implementation

{ TConnectedUserList procedures }

function TConnectedUserList.Add(Value: TConnectedUser): PConnectedUser;
var
 data: PConnectedUser;
begin
 data:=AllocMem(sizeof(TConnectedUser));
 with data^ do
 begin
  Username := Value.Username;
  Password := Value.Password;
  RemoteIP := Value.RemoteIP;
  Remoteport := Value.Remoteport;
  Speed := Value.Speed;
  Sharedbytes := Value.Sharedbytes;
  Description := Value.Description;
  Email := Value.Email;
  Status := Value.Status;
  Socket := Value.Socket;
  SuccessLoggedIn := Value.SuccessLoggedIn;
  BadLogin := Value.BadLogin;
  ValidateDenide := Value.ValidateDenide;
  Passwordauthoriseinprogress := Value.Passwordauthoriseinprogress;
  lock := Value.lock;
  key := Value.key;
  keyauthorisedone := Value.keyauthorisedone;
  Pointer(buffer) := nil;
  buffer := Value.buffer;
 end;
 inherited Add(data);
 Result:=data;
end;

procedure TConnectedUserList.Clear;
begin
 while count>0 do
  Delete(count-1);
 inherited;
end;

procedure TConnectedUserList.Delete(socket:TIdPeerThread);
var i:integer;
    n:integer;
begin
 n := -1;
 for i := 0 to count -1 do
 begin
  if socket = PConnectedUser(Items[i])^.Socket then n := i;
 end;
 if n <> -1 then delete(n);
end;

procedure TConnectedUserList.Delete(Index: Integer);
begin
 if (Index<0) or (Index>=Count) then exit;
 if Items[Index]<>nil then
 begin
  Finalize(PConnectedUser(Items[Index])^);
  FreeMem(PConnectedUser(Items[Index]),sizeof(TConnectedUser));
 end;
 inherited;
end;

constructor TConnectedUserList.Create;
begin
 inherited;
end;

destructor TConnectedUserList.Destroy;
begin
 Clear;
 inherited;
end;


end.
